#if !defined(__parse_options_hpp__)
#define __parse_options_hpp__



enum po_arg_type {
	poatVoid, poatNone, poatBoolean, poatInt, poatOptionalInt, poatFloat, poatOptionalFloat, poatString, poatOptionalString
};

enum po_arg_src {
	poasVoid, poasOptCfg, poasOptOnly, poasCfgOnly
};

enum { po_string_size = 63 };


// This is what the user's program must declare an array of.
struct po_declared_option
{
	int nbr;
	po_arg_src arg_src;
	po_arg_type arg_type;
	char char_name;
	char long_name[po_string_size + 1];
};



/*
	This is what parse_option fills out and returns in the options_array.  Check nbr to determine what option this struct is for.  If
	its one of the poatOptionalXXX options, check arg_is_present to see	if an arg was passed with it.  If its a config option, lineno
	will be set to the line it was found on.  If its an integer option, the argument to the option is in arg.int_val, boolean options
	are in arg.bool_val and so on.
*/
struct po_returned_option
{
	int nbr;
	bool arg_is_present;
	unsigned lineno;
	union {
		bool bool_val;
		long int_val;
		double float_val;
		char string_val[po_string_size + 1];
	} arg;
};



//-----------------------------------------------------------------------------
class parse_options
{

public:
	void parse_cfgs(string file, po_declared_option* opts);
	void parse_cfgs(char* file, po_declared_option* opts);

	void parse_args(int argc, char** argv, po_declared_option* opts);

	vector<po_returned_option> options_array;
	vector<string> arguments_array;

protected:
	enum po_reading_type { portConfigs, portOptions };

	void parse_long_option(string& src);
	void parse_short_option(string& src);
	void parse_cfgs_readcfg();

	void validate_long_option();
	void validate_short_option();

	void read_int_arg(const char* dashes);
	void read_float_arg(const char* dashes);
	void read_string_arg(const char* dashes);

	po_declared_option* dopts;
	po_returned_option ropt;

	unsigned lineno;
	file_info cf;

	string opt;
	string arg;

	po_reading_type reading;
};



#endif /* !defined(__parse_options_hpp__) */
